# encoding=utf-8

class TemplateConfig(object):

    def __init__(self):
        return self
    
    @property
    # 离线标签 / 准实时
    def tag_type(self):
        return self.tag_type
    
    # （小微/消费/经营/其他/迁移）
    def tag_belong(self):
        return self.tag_belong

    # 自定义 Name
    def tag_name(self):
        return self.tag_name
    
    # tag_code，非空
    def tag_code(self):
        return self.tag_code
    
    # 自定义版本号， 默认使用1.0.0
    def tag_version(self):
        if isinstance(self.tag_version, str) and self.tag_version != '':
            return self.tag_version
        else:
            return "1.0.0"
    
    # 标签优先级
    def tag_priority(self):
        return self.tag_priority
    
    # 数据层级
    def tag_data_level(self):
        return self.tag_data_level
    
    # 数据依赖项
    def tag_dependencies(self):
        return self.tag_dependencies
    
    # 实体类型，依赖子类实现
    def tag_entity_type(self):
        raise RuntimeError("未在子类指定存储目标地址")

    # 存储目的地
    def tag_destination(self):
        raise RuntimeError("未在子类指定存储目标地址")



# encoding=utf-8
import os
from service_engine import service_manager as ServiceManager


def deal_item_path():
    # 处理下基础路径
    current_file_path = os.path.abspath(__file__)
    # 获取当前Python文件的父路径
    parent_directory = os.path.dirname(current_file_path)
    # 获取父路径的父路径
    grandparent_directory = os.path.dirname(parent_directory)
    os.chdir(grandparent_directory)


if __name__ == '__main__':
    deal_item_path()
    print('=================================================================')
    print('【帮助文档】\n[当前项目根路径的绝对路径]：{} \n[跳]：请使用回车键'.format(os.getcwd()))
    print('=================================================================')
    ServiceManager.start_constructor_tag_template()



# encoding=utf-8

import os, re, platform

# def create_tag

def start_construct_tag_template(config_dict):
    template_type = config_dict['template_type']
    
    if os.path.isdir("dxm"):
        code_base_path="/dxm/loan-data/loan-data-warehouse-python"
    elif os.path.isdir("loan-data"):
        code_base_path="/loan-data/loan-data-warehouse-python"
    elif os.path.isdir("loan-data-warehouse-python"):
        code_base_path="/loan-data-warehouse-python"
    else:
        code_base_path="/dxm/loan-data/loan-data-warehouse-python"

    tempalte_path = code_base_path+'/hugsqltools/template_engine/tag_templates/offline'
    if template_type == 1:
        tempalte_path = code_base_path+'/hugsqltools/template_engine/tag_templates/offline'
    elif template_type == 2:
        tempalte_path = code_base_path+'/hugsqltools/template_engine/tag_templates/nrt'
    else :
        raise RuntimeError('程序逻辑异常，请联系chaiyuan_dxm排查')
    
    # base_path = os.getcwd()
    base_path=code_base_path

    # 加上操作系统的判断
    if platform.system() == 'Windows':
        # base_path = base_path.replace("\\", "/")    # // ---> /
        final_path = tempalte_path
        output_path = '%s/dw/%s/tag/%s/%s' % (base_path, config_dict['tag_data_level'], config_dict['append_path'],config_dict['tag_code'])
    else:
        final_path = tempalte_path
        output_path = '%s/dw/%s/tag/%s/%s' % (base_path, config_dict['tag_data_level'], config_dict['append_path'],config_dict['tag_code'])
    

############################################################################################################################
    ######################### 调试模块 #########################
    # final_path='/dxm/loan-data/loan-data-warehouse-python/hugsqltools/template_engine/tag_templates/nrt'
    # output_path='/dxm/loan-data/loan-data-warehouse-python/dw/ads/tag/index_06_nrt/%s' %(config_dict['tag_code'])
    
    print('final_path',final_path)
    print('output_path',output_path)
    # index=0
    # for dirpath, dirnames, filenames in os.walk(final_path):
    #     for filename in filenames:
    #         print(index,dirpath,filename)
    #         index+=1
############################################################################################################################
    if not os.path.exists(output_path):
        os.makedirs(output_path)
    for dirpath, dirnames, filenames in os.walk(final_path):
        for filename in filenames:

            temp_dependencies = ''
            pre_rules = []
            for dependency in config_dict['tag_dependencies']:
                table = dependency['table']
                partitions=dependency['partitions']
                partitions_name = dependency['partitions_name']
                temp_dependencies += '  - table: %s\n'%(table)
                temp_dependencies += "    partitions: '%s'\n"%(partitions)
                pre_rule = "SELECT if(count(1)>1, 0, 1) AS res \nFROM hivedgs.%s \nWHERE \n%s='%s'"%(table, partitions_name,partitions)
                pre_rules.append(pre_rule)
            
            batch_staticize_codes=[]

            if 'tag_staticize_codes' in config_dict.keys():    
                for code_item in config_dict['tag_staticize_codes']:
                    statistic_code=code_item['tag_staticize_code']
                    batch_staticize_codes.append(statistic_code)

            tag_entity_key = 'did'

            tag_entity_type = config_dict['tag_entity_type'].lower()
            if tag_entity_type == 'did':
                tag_entity_key = 'did'
            elif tag_entity_type == 'cmid':
                tag_entity_key = 'customer_id'

            temp_staticize_codes=''
            if len(batch_staticize_codes)>0:
                temp_staticize_codes = "\
outputs: \n\
  table: ads_loan_user_tag_df \n\
  dws_table: false \n\
  staticize_vars_config: { \n\
      'entity_type':'%s', \n\
      'partition_config': { \n\
      'ads_day':'{DATE-1}', \n\
      'entity_type':'%s', \n\
      'tag_code':'construct_code',\n\
      'version':'1.0.0' \n\
      },\n\
      'tag_code': [\n\
      %s\n\
      ]\n\
  }"%(tag_entity_type, tag_entity_type, "'"+"','".join(batch_staticize_codes)+"'")

            pre_check_sql = 'WITH '
            rule_index = 1
            rule_length = len(pre_rules)
            poly_rules = ''

            for pre_rule in pre_rules:
                rule_name = 'rule%s'%(str(rule_index))
                rule_item = '%s AS (\n%s\n)'%(rule_name, pre_rule)
                if rule_index < rule_length:
                    rule_item += ','
                    poly_rules += 'SELECT res FROM %s \nUNION ALL \n'%(rule_name)
                else:
                    poly_rules += 'SELECT res FROM %s'%(rule_name)
                pre_check_sql += '%s \n'%(rule_item)
                rule_index += 1
            
            pre_check_sql += "SELECT if(sum(res) = 0 ,'true','false') AS res \nFROM (\n%s\n)"%(poly_rules)

            replace_config_dict = {
                'tag_code': config_dict['tag_code'],
                'tag_name': config_dict['tag_name'],
                'tag_dependencies': temp_dependencies,
                'tag_version': config_dict['tag_version'],
                'tag_entity_type': config_dict['tag_entity_type'],
                'tag_priority': config_dict['tag_priority'],
                'tag_belong_path': config_dict['append_path'],
                'tag_entity_key': tag_entity_key,
                'pre_check_sql': pre_check_sql,
                # 'tag_rccgroup': config_dict['tag_rccgroup'],
                'processing_owner': config_dict['processing_owner'],
                # 'apply_scenario': config_dict['apply_scenario'],
                'tag_staticize': config_dict['tag_staticize'],
                'tag_staticize_codes': temp_staticize_codes
            }

            if platform.system() == 'Windows':
                temp_filepath = os.path.join(dirpath, filename).replace("\\", "/")
            else:
                temp_filepath = os.path.join(dirpath, filename)
            # print('temp_filepath',temp_filepath)
            # 指定读写文件的编码方式
            with open(temp_filepath, encoding="utf-8") as file_obj:
                file_content = file_obj.read()

                final_rs = file_content
                for key,value in replace_config_dict.items():
                    regex = r'({{[ ]*%s[ ]*}})'%(key)
                    final_rs = re.sub(regex, value, final_rs)

                output_filepath = '%s/%s'%(output_path, filename.replace('tagcode', config_dict['tag_code']))

                with open(output_filepath, 'w', encoding="utf-8") as write_obj:
                    write_obj.write(final_rs)
                    write_obj.close()




import subprocess


def get_git_user_name():
    user_name = 'None'
    try:
        work = subprocess.Popen(['git', 'config', 'user.name'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdout, stderr = work.communicate()
        if work.returncode != 0:
            print(format(stderr))
        if isinstance(stdout, str):
            user_name = stdout
        elif isinstance(stdout, bytes):
            user_name = stdout.decode('utf-8')
        else:
            print(format(stdout))

    except Exception as err:
        print(format(err.message))
    return user_name


if __name__ == '__main__':
    name = get_git_user_name()
    print(name)



from ads_config import AdsTagConfig

def prepare_tag_config(dict:dict):
    print('ads tag config info %s'%(dict))
    tag_config = AdsTagConfig()
    # 归属
    tag_config.tagt_data_level = 'ads'
    # 业务归属
    tag_config.tag_belong = dict['tag_belong']
    # 标签Code
    tag_config.tag_code = dict['tag_code']
    # 标签Name
    tag_config.tag_name = dict['tag_name']
    # 优先级
    tag_config.tag_priority = dict['tag_priority']
    # 实体类型
    tag_config.tag_priority = dict['tag_entity']
    # 数据依赖
    tag_config.tag_dependencies = dict['tag_dependencies']


# encoding = utf-8

class TemplateConfig(object):
    def __init__(self, config_dict):
        self.config_dict = config_dict

class TagTemplateConfig(TemplateConfig):
    def __init__(self, config_dict):
        super().__init__(config_dict)


# encoding=utf-8

import os,platform

from input_engine import input_manager as Input
from config_engine import config_manager as Config

from tag_template_cons.tag_template_constants import TagConstantsTips, TagMultipleConstantsTips

from template_engine import template_manager as TemplateManager

from utils_engine import git_utils

# 创建交互式配置
def interactive_tag_config():

    tag_configs = []
    
    tag_name_config = TagConstantsTips(
        tag_tip='请输入标签中文名:',
        tag_tip_name='tag_name'
    )
    tag_configs.append(tag_name_config)

    tag_code_config = TagConstantsTips(
        tag_tip='请输入标签Code:',
        tag_duplicate_check=True,
        tag_tip_name='tag_code'
    )
    tag_configs.append(tag_code_config)

    # 标签默认ADS层
    # tag_datalevel_config = TagConstantsTips(
    #     tag_tip="请输入数据层级(ads/ods/dws/dwd/dim):",
    #     tag_options=['ads','ods','dws','dwd','dim'],
    #     tag_tip_name='tag_data_level'
    # )
    # tag_configs.append(tag_datalevel_config)
    
    tag_belong_config = TagConstantsTips(
        tag_tip='请输入标签目录归属(1-消费[常规标签]/2-小微/3-其他经营[广告/生态/线下]/4-经营时机/5-历史标签[迁移]/6-准实时复算/7-消费宽表/8-业务自助加工/9-风控策略):',
        tag_options=["1", "2", "3", "4", "5", "6", "7", "8", "9"],
        tag_tip_name='tag_belong'
    )
    tag_configs.append(tag_belong_config)

    tag_version_config = TagConstantsTips(
        tag_tip='请输入标签版本信息(跳过默认使用1.0.0):',
        tag_nullable=True,
        tag_tip_name='tag_version',
        tag_defalut_rs='1.0.0'
    )
    tag_configs.append(tag_version_config)

    tag_entitytype_config = TagConstantsTips(
        tag_tip='请输入标签实体类型：[可选项:did/cmid/adjust_id](跳过默认使用did):',
        tag_nullable=True,
        tag_defalut_rs='did',
        tag_tip_name='tag_entity_type'
    )
    tag_configs.append(tag_entitytype_config)

    tag_staticize_config = TagConstantsTips(
        tag_tip='请输入是否要加工成静态变量：[可选项:false/true](跳过默认不入库：使用false):',
        tag_nullable=True,
        tag_tip_name='tag_staticize',
        tag_defalut_rs='false'
    )
    tag_configs.append(tag_staticize_config)

    # 加工的RCC组目前确定就会统一一个，去掉相关逻辑
    # tag_rccgroup_config = TagConstantsTips(
    #     tag_tip='请输入标签加工的RCC组：[可选项:fsg_rcc_loan_data_rd/fsg_rcc_loan_data_qualified_rd](跳过默认使用fsg_rcc_loan_data_rd):',
    #     tag_nullable=True,
    #     tag_tip_name='tag_rccgroup',
    #     tag_defalut_rs='fsg_rcc_loan_data_rd'
    # )
    # tag_configs.append(tag_rccgroup_config)

    # 标签的应用场景目前根据，优先级和消费宽表进行自助判断；后续场景变多在进行处理
    # tag_apply_scenario_config = TagConstantsTips(
    #     tag_tip='请输入标签应用场景：[可选项:fsg_rcc_loan_data_rd/fsg_rcc_loan_data_qualified_rd](跳过默认使用fsg_rcc_loan_data_rd):',
    #     tag_nullable=True,
    #     tag_tip_name='tag_rccgroup',
    #     tag_defalut_rs='fsg_rcc_loan_data_rd'
    # )
    # tag_configs.append(tag_apply_scenario_config)

    tag_priority_config = TagConstantsTips(
        tag_tip='是否为高优标签[高优使用P0](跳过默认使用P1)',
        tag_tip_name='tag_priority',
        tag_nullable=True,
        tag_defalut_rs='P1'
    )
    tag_configs.append(tag_priority_config)

    tag_dependencies_config = TagMultipleConstantsTips(
        tag_tip_name='tag_dependencies',
        tag_constants=[],
        tag_complete = TagConstantsTips(
            tag_tip='是否已完成依赖源输入(Y/N):',
            tag_options=['Y', 'N'],
            tag_tip_name='tag_dependencies_complete',
            tag_multilpeable = True,
            tag_complete_flag='Y'
        )
    )

    tag_dependtable_config = TagConstantsTips(
        tag_tip='请输入依赖数据源(dbname.tablename):',
        tag_tip_name='table'
    )
    tag_dependpartitions_config = TagConstantsTips(
        tag_tip='请输入依赖分区[{DATE-1}/{DATE2-1}/{DATE-1}23](跳过默认为{DATE-1}):',
        tag_tip_name='partitions',
        tag_nullable=True,
        tag_defalut_rs='{DATE-1}'
    )
    tag_dependpartitionsname_config = TagConstantsTips(
        tag_tip='请输入依赖分区[ads_day/dws_day/dwd_day/dim_day/ods_day/dwd_hour](跳过默认为dwd_day):',
        tag_tip_name='partitions_name',
        tag_nullable=True,
        tag_defalut_rs='dwd_day'
    )
    tag_dependencies_config.tag_constants.append(tag_dependtable_config)
    tag_dependencies_config.tag_constants.append(tag_dependpartitions_config)
    tag_dependencies_config.tag_constants.append(tag_dependpartitionsname_config)

    tag_configs.append(tag_dependencies_config)
    
    config_dict = {}
    for tag_config in tag_configs:
        input_rs = Input.start_input_tips(tag_config)
        print(tag_config.tag_tip_name)
        if tag_config.tag_duplicate_check == True:
            check_rs = tag_flag_duplicate_check(input_rs)
            if check_rs == True:
                raise RuntimeError('录入 %s 重复，请检查后重新录入'%(tag_config.tag_tip_name))
        config_dict[tag_config.tag_tip_name] = input_rs

    if config_dict['tag_staticize']==True or config_dict['tag_staticize']=='true':

        batch_config = TagConstantsTips(
            tag_tip='是否批量录入静态变量(Y/N):',
            tag_tip_name='batch_input_static_vars',
            tag_nullable=True,
            tag_defalut_rs='N'
        )

        batch_input_rs = Input.start_input_tips(batch_config)

        if batch_input_rs == 'Y':
            tag_config = TagMultipleConstantsTips(
                tag_tip_name='tag_staticize_codes',
                tag_constants=[],
                tag_complete = TagConstantsTips(
                    tag_tip='是否已完成静态变量录入(Y/N):',
                    tag_options=['Y', 'N'],
                    tag_tip_name='tag_staticize_codes_complete',
                    tag_multilpeable = True,
                    tag_complete_flag='Y'
                )
            )
            tag_staticize_code_config = TagConstantsTips(
                tag_tip='请输入静态变量code:',
                tag_tip_name='tag_staticize_code',
                tag_nullable=False,
            )
            tag_staticize_code_name_config = TagConstantsTips(
                tag_tip='请输入静态变量name:',
                tag_tip_name='tag_staticize_code_name',
                tag_nullable=False,
            )
            tag_config.tag_constants.append(tag_staticize_code_config)
            tag_config.tag_constants.append(tag_staticize_code_name_config)
            input_rs = Input.start_input_tips(tag_config)
            config_dict['tag_staticize_codes']=input_rs
    
    return config_dict

def tag_flag_duplicate_check(tag_flag):

    base_path = os.getcwd()

    tmp_base_path = '../../dw/ads/tag'

    # 加上操作系统的判断
    if platform.system() == 'Windows':
        base_path = base_path.replace("\\", "/")    # // ---> /
        output_path = '%s/dw/ads/tag/' % (base_path)
    else:
        base_path = base_path.replace("\\", "/")    # // ---> /
        output_path = '%s/dw/ads/tag/' % (base_path)

    is_duplicate = False

    for dirpath, dirnames, filenames in os.walk(output_path):
        if dirpath == tag_flag:
            is_duplicate = True
            break
        for filename in filenames:
            if filename == tag_flag or filename == '%s.sql'%(tag_flag):
                is_duplicate = True
                break
        if is_duplicate == True:
            break
    return is_duplicate

# 触发template task
def resume_template_task(config_dict):
    # template_type = 1/离线标签  2/准实时标签
    template_type = 1
    tag_belong = config_dict['tag_belong']
    
    if not isinstance(tag_belong, str) or len(tag_belong) == 0 :
        raise RuntimeError('代码逻辑异常，请联系chaiyuan_dxm排查问题')
    
    __int_tag_belong = int(tag_belong)

    append_belong_path = ''
    # 标签的应用场景，0-普通筛客，1-消费宽表，2-人工电召
    # 默认全部为0-普通筛客，当优先级为高优时可用于人工电召，当放到消费目录下则用于消费宽表
    apply_scenario_value = "  - '0'\n"

    if __int_tag_belong == 1:
        template_type = 1
        append_belong_path = 'index_01_consumption'
    elif __int_tag_belong == 2:
        template_type = 1
        append_belong_path = 'index_02_xiaowei'
    elif __int_tag_belong == 3:
        template_type = 1
        append_belong_path = 'index_03_other'
    elif __int_tag_belong == 4:
        template_type = 1
        append_belong_path = 'index_04_bussiness_operation'
    elif __int_tag_belong == 5:
        template_type = 1
        append_belong_path = 'index_05_history'
    elif __int_tag_belong == 6:
        template_type = 2
        append_belong_path = 'index_06_nrt'
    elif __int_tag_belong == 7:
        template_type = 1
        append_belong_path = 'dws_loan_ccl_tag'
        apply_scenario_value = apply_scenario_value + "  - '1'\n"
    elif __int_tag_belong == 8:
        template_type = 1
        append_belong_path = 'index_08_business_self'
    elif __int_tag_belong == 9:
        template_type = 1
        append_belong_path = 'index_09_ris'
    else:
        raise RuntimeError('不支持的业务归属，请联系chaiyuan_dxm修改')

    # 处理高优标签默认可用于电召场景
    tag_priority = config_dict['tag_priority']
    if tag_priority == 'P0':
        apply_scenario_value = apply_scenario_value + "  - '2'\n"

    data_level = 'ads'

    template_config_dict = config_dict
    template_config_dict['tag_data_level'] = data_level
    template_config_dict['template_type'] = template_type
    template_config_dict['append_path'] = append_belong_path
    template_config_dict['base_path'] = '../../dw/%s/tag'%(data_level)

    # processing_owner = git_utils.get_git_user_name()
    processing_owner='xxx_dxm'

    template_config_dict['processing_owner'] = processing_owner
    # template_config_dict['apply_scenario'] = apply_scenario_value

    TemplateManager.start_construct_tag_template(template_config_dict)


def start_constructor_tag_template():
    # template type support offline_tag and nrt_tag
    config_dict = interactive_tag_config()
    resume_template_task(config_dict)



# encoding=utf-8

from constants_engine.constants import Constants
from utils_engine import core_utils as CoreUtils

kRepeatTipContent = '输入内容不符合要求，请重新输入:'
kMultipleTipContent = '是否继续输入(Y/N):'
kMultipleTipOptions = ['Y','N']

class TagConstants(Constants):
    def __init__(self):
        return self

class TagConstantsTips(TagConstants):

    def __init__(self, 
                 tag_tip, 
                 tag_tip_name,
                 tag_options=[], 
                 tag_nullable=False, 
                 tag_multilpeable=False,
                 tag_defalut_rs='',
                 tag_complete_flag='Y',
                 tag_duplicate_check=False,
                 tag_repeat_tip=kRepeatTipContent):
        
        self.tag_tip = tag_tip
        self.tag_nullable = tag_nullable
        self.tag_options = CoreUtils.safe_list(tag_options)
        self.tag_repeat_tip = tag_repeat_tip
        self.tag_multilpeable = tag_multilpeable
        self.tag_tip_name = tag_tip_name
        self.tag_complete_flag = tag_complete_flag
        self.tag_defalut_rs = tag_defalut_rs
        self.tag_duplicate_check = tag_duplicate_check
    
# 多次录入提醒
class TagMultipleConstantsTips(Constants):

    def __init__(self, 
                 tag_constants, 
                 tag_tip_name, 
                 tag_complete,
                 tag_duplicate_check=False
                 ):
        
        self.tag_constants = CoreUtils.safe_list(tag_constants)
        self.tag_complete = CoreUtils.obj_null_filter(tag_complete, TagConstantsTips)
        self.tag_tip_name = tag_tip_name
        self.tag_duplicate_check = tag_duplicate_check




from constants_engine.constants import Constants

class InputTipConstants(Constants):
    @property
    def tip_name(self):
        return self.tip_name
    
    def tip_options(self):
        return self.tip_options
    

class InputRepeatTipConstants(InputTipConstants):
    @property
    def tip_name(self):
        return "请重新输入："

    def tip_options(self):
        return super().tip_options()